//  Beispiel 16.2: MQTT-Allnet Daten vom Server holen und anzeigen
//
//  Unter Verwendung der IoT Brick Library

#include <all_IoT.h>
#include <all_MQTT.h>


int counter = 0;


void setup()
{
  IoT_Init(true);
  IoT_TerminalWaitInit(true);
  IoT_WLANautoConnect(true);                                        // Verbindung, ggf. gespeicherte, über WLAN herstellen
  IoT_NTPinit();                                                    // NTP-Timeserver initialisieren
  
  IoT_DisplayClear(24);                                             // OLED Display löschen (24 Punkt Schrift)
  IoT_DisplayAlignText(TEXT_ALIGN_CENTER);                          // Zentrierte Darstellung des Textes
  IoT_DisplayDrawText(64, 5, "Suche mqtt");
  IoT_DisplayDrawText(64, 34, "Server...");
  IoT_DisplayUpdate();                                              // OLED-Anzeige aktualisieren
  Serial.println("");

  // MQTT_Init(IPval("212.18.29.166"), 1883, "BrickRknowledge", "open", "Allnet"); // iot.allnet.de = IPval("212.18.29.166") ("open",  "Allnet")
  // MQTT_Connect("general/data/#");

  MQTT_Init("iot.allnet.de", 1883, "BrickRknowledge", "open", "Allnet"); // iot.allnet.de = IPval("212.18.29.166") ("open",  "Allnet")
  MQTT_Connect("general/data/#");

  Serial.print(t_TerminalClearScreen());
  MQTT_ConnectTrace = false;
}

void loop()
{
  IoT_Idle();

  if (IoT_Keypress())                                               // Wenn Taster gedrückt wird, Konfiguration zeigen
  {
    IoT_DisplayWLANstatus();
    IoT_DisplayUpdate();                                            // OLED-Anzeige aktualisieren
    Serial.println(IoT_NTPdatetime() + " (" + IoT_NTPdaylightSavingTime(true) + ")");
    Serial.print("WiFi is ");
    Serial.print(IoT_WLANconnected() ? "connected" : "not connected");
    Serial.println(". Uptime: " + IoT_WLANuptime(true) + " seit " + IoT_WLANstartDatetime());
    IoT_WaitNoKeypress();
  }
  else
  {
    IoT_Idle();
    if (counter % 25 == 0) //Infos seriell nicht zu oft ausgeben (ca. alle 2 Sekunden)
    {
      if (IoT_NTPeventAvail)                                        // Zeitevent triggern
      {
        IoT_NTPprintEvent(IoT_NTPcurrentEvent);
        IoT_NTPeventAvail = false;
      }
    }
    
    IoT_DisplayClear(16);                                           // OLED Display löschen (16 Punkt Schrift)
    IoT_DisplayAlignText(TEXT_ALIGN_CENTER);                        // Zentrierte Darstellung des Textes
    if (IoT_WLANinited)
    {
      IoT_DisplayClear(24);                                         // OLED Display löschen (24 Punkt Schrift)
      if (MQTT_Client.connected())
      {
        IoT_DisplayAlignText(TEXT_ALIGN_CENTER);                    // Zentrierte Darstellung des Textes
        IoT_DisplayDrawText(64, 5, "Verbunden");
        IoT_DisplayDrawText(64, 34, "mit mqtt");
      }
      else
      {
        IoT_DisplayAlignText(TEXT_ALIGN_CENTER);                    // Zentrierte Darstellung des Textes
        IoT_DisplayDrawText(64, 5, "Suche mqtt");
        IoT_DisplayDrawText(64, 34, "Server...");
      }
      IoT_DisplayUpdate();                                          // OLED-Anzeige aktualisieren
    }
    else
    {
      IoT_DisplayClear(24);                                         // OLED Display löschen (24 Punkt Schrift)
      IoT_DisplayAlignText(TEXT_ALIGN_CENTER);                      // Zentrierte Darstellung des Textes
      IoT_DisplayDrawText(64, 5, "Kein");
      IoT_DisplayDrawText(64, 34, "WLAN");
    }
  }
  IoT_DisplayUpdate();                                              // OLED-Anzeige aktualisieren

  bool success = MQTT_HandleClient();                               // Verbindung aufrecht erhalten, ggf. neu aufbauen

  if (MQTT_EventAvail)
  {
    MQTT_GetNextEvent();
    if (not(MQTT_AllnetParseEventCurrent()))
    {
      Serial.print("Time: "      + cleanasc(String(MQTT_EventCurrentTime)));
      Serial.println(", Topic: " + cleanasc(String(MQTT_EventCurrentTopic)));
      Serial.println("Message: " + String(MQTT_EventCurrentMessage));
    }
  }
  else
  {
    String eol = t_TerminalClearEndOfLine();
    t_TerminalRead();                                                     // Reset-Anforderung prüfen
    Serial.print(t_TerminalCursorHome());
    Serial.print("Verbindungsstatus mit Server '" + MQTT_ServerURL + "': ");
    IoT_DisplayClear(24);                                                 // OLED Display löschen (24 Punkt Schrift)
    if (MQTT_Client.connected())
    {
      IoT_DisplayAlignText(TEXT_ALIGN_CENTER);                            // Zentrierte Darstellung des Textes
      IoT_DisplayDrawText(64, 5, "Verbunden");
      IoT_DisplayDrawText(64, 34, "mit mqtt");
      Serial.println("Verbindung hergestellt" + eol);
    }
    else
    {
      IoT_DisplayAlignText(TEXT_ALIGN_CENTER);                            // Zentrierte Darstellung des Textes
      IoT_DisplayDrawText(64, 5, "Suche mqtt");
      IoT_DisplayDrawText(64, 34, "Server...");
      Serial.println("Server wird gesucht" + eol);
    }
    IoT_DisplayUpdate();                                                  // OLED-Anzeige aktualisieren
    Serial.println("");
    Serial.print  (fillcenter("- Temperaturen ", "-", 80));
    Serial.println(eol);
    Serial.print  ("Hamburg     = " + strrealform(MQTT_AllnetTemperaturHamburg,      32, 2, 1, false, false) + "°C" + spc(19));
    Serial.println("Berlin      = " + strrealform(MQTT_AllnetTemperaturBerlin,       32, 2, 1, false, false) + "°C" + eol);
    Serial.print  ("Frankfurt   = " + strrealform(MQTT_AllnetTemperaturFrankfurt,    32, 2, 1, false, false) + "°C" + spc(19));
    Serial.println("Stuttgart   = " + strrealform(MQTT_AllnetTemperaturStuttgart,    32, 2, 1, false, false) + "°C" + eol);
    Serial.print  ("Hannover    = " + strrealform(MQTT_AllnetTemperaturHannover,     32, 2, 1, false, false) + "°C" + spc(19));
    Serial.println("München     = " + strrealform(MQTT_AllnetTemperaturMuenchen,     32, 2, 1, false, false) + "°C" + eol);
    Serial.print  ("Köln        = " + strrealform(MQTT_AllnetTemperaturKoeln,        32, 2, 1, false, false) + "°C" + spc(19));
    Serial.println("Düsseldorf  = " + strrealform(MQTT_AllnetTemperaturDuesseldorf,  32, 2, 1, false, false) + "°C" + eol);
    Serial.print  ("Nürnberg    = " + strrealform(MQTT_AllnetTemperaturNuernberg,    32, 2, 1, false, false) + "°C" + spc(19));
    Serial.println("Dresden     = " + strrealform(MQTT_AllnetTemperaturDresden,      32, 2, 1, false, false) + "°C" + eol);
    Serial.println(eol);
    Serial.print  (fillcenter(" Aktien- und Währungskurse ", "-", 81)); // 81 weil das "ä" längentechnisch 2 Zeichen belegt
    Serial.println(eol);
    Serial.print  ("DAX         = " + strrealform(MQTT_AllnetFinanceDaxVal,          32, 6, 2, true, false)  + spc(16));
    Serial.print  ("Differenz   = " + strrealform(MQTT_AllnetFinanceDaxDifVal,       32, 1, 2, false, false, true) + " (");
    Serial.println(                   strrealform(MQTT_AllnetFinanceDaxDifPerc,      32, 1, 2, false, false, true) + "%)" + eol);
    Serial.print  ("MDAX        = " + strrealform(MQTT_AllnetFinanceMDaxVal,         32, 6, 2, true, false)  + spc(16));
    Serial.print  ("Differenz   = " + strrealform(MQTT_AllnetFinanceMDaxDifVal,      32, 1, 2, false, false, true)  + " (");
    Serial.println(                   strrealform(MQTT_AllnetFinanceMDaxDifPerc,     32, 1, 2, false, false, true) + "%)" + eol);
    Serial.print  ("ESTX50      = " + strrealform(MQTT_AllnetFinanceEStx50Val,       32, 6, 2, true, false)  + spc(16));
    Serial.print  ("Differenz   = " + strrealform(MQTT_AllnetFinanceEStx50DifVal,    32, 1, 2, false, false, true)  + " (");
    Serial.println(                   strrealform(MQTT_AllnetFinanceEStx50DifPerc,   32, 1, 2, false, false, true) + "%)" + eol);
    Serial.print  ("Gold        = " + strrealform(MQTT_AllnetFinanceGoldVal,         32, 6, 2, true, false)  + spc(16));
    Serial.print  ("Differenz   = " + strrealform(MQTT_AllnetFinanceGoldDifVal,      32, 1, 2, false, false, true)  + " (");
    Serial.println(                   strrealform(MQTT_AllnetFinanceGoldDifPerc,     32, 1, 2, false, false, true) + "%)" + eol);
    Serial.print  ("Öl          = " + strrealform(MQTT_AllnetFinanceOelVal,          32, 6, 2, true, false)  + spc(16));
    Serial.print  ("Differenz   = " + strrealform(MQTT_AllnetFinanceOelDifVal,       32, 1, 2, false, false, true)  + " (");
    Serial.println(                   strrealform(MQTT_AllnetFinanceOelDifPerc,      32, 1, 2, false, false, true) + "%)" + eol);
    Serial.print  ("EUR/USD     = " + strrealform(MQTT_AllnetFinanceEURVal,          32, 6, 2, true, false)  + spc(16));
    Serial.print  ("Differenz   = " + strrealform(MQTT_AllnetFinanceEURDifVal,       32, 1, 2, false, false, true)  + " (");
    Serial.println(                   strrealform(MQTT_AllnetFinanceEURDifPerc,      32, 1, 2, false, false, true) + "%)" + eol);
    Serial.print  ("BitCoin/USD = " + strrealform(MQTT_AllnetFinanceBTCVal,          32, 6, 2, true, false)  + spc(16));
    Serial.print  ("Differenz   = " + strrealform(MQTT_AllnetFinanceBTCDifVal,       32, 1, 2, false, false, true)  + " (");
    Serial.println(                   strrealform(MQTT_AllnetFinanceBTCDifPerc,      32, 1, 2, false, false, true) + "%)" + eol);
    Serial.println(eol);
    Serial.print  (fillcenter(" Sonstige ", "-", 80));
    Serial.println(eol);
    delay(1000);                                                          // 1 Sekunde warten
  }

  counter++;
}

